% This Code simulates the temperatures for Ulmatec Pyro Double-Pipe design
% Coded by Lene sy 30.03.2017

% The pipe is modeled using finite difference method (FDM)
% For heat transfer Lumped Capacitance Method (LCM) is used 

clear all
clc
clf

%% INPUT Parameters=======================================================

m=400;              %[-] number of elements along L
Tin=20+273.15;      %[K] Tin water inlet temperature
Tlim=3+273.15;      %[K] Tlim is the surface temperature limmit 
                    %from DNV GL (+3degC or 276.15K)
Tinf=-10+273.15;    %[K] air temperature 
Vflow=0.01/3600;    %[m^3/s] volumetric flow
U=0;                %[m/s] wind velocity
dr=0.6565;          %[-] radius factor %0.8388 50x38, %0.6565 50x32
r2=0.025;           %[m] radius outer pipe %0.025 for {\O}50mm
L=2.5;              %[m] length of the pipe

%% CALCULATION STARTS ====================================================

% Initializing Parameters-------------------------------------------------

%Parameters dependent on INPUT
dL=L/m;             %[m] Length of each element
r1=dr*r2;           %[m] radius inner pipe

%Making zero vectors (Ax-B=0) => (Q*T = Qtot)
n=2;                %This is a fixed parameter
Q=zeros(m*n,m*n);   %A-matrix
Qtot=zeros(m*n,1);  %B-matrix

%Since Q matrix is dependent on the T vector, an iteration process has to
                    %be done, here T0 is the startig condition.
for e=1:(n*m)
T0(e,1)=Tin;        %Starting iteration with Tin for all elements.
end

% Iteration starts--------------------------------------------------------

for cal=1:40        %iteration loop, updating the matrixes for each new T0 

% Making the matrixes A and B calling the functions QL, QR1 and QR22:
    %QL=QL(Vflow,T1) Thermal Capacitance coeff. [W/K], mass transport
    %QR1=QR1(r1,r2,T1,T2,dL) Heat transfer coeff. [W/K],inner pipe wall
    %QR22(r1,r2,T2,Tinf,dL,U) Heat transfer coeff. [W/K], outer pipe wall

% Making the boundary heat flow matrix B
for in=1:m
    Qtot(1,1)=-QL(Vflow,Tin)*Tin; %[W] Heat transfer from the inlet flow
    Qtot(in*n,1)=-QR22(r1,r2,T0(in*n,1),Tinf,dL,U,Vflow)*(Tinf); 
                    %[W] Heat transfer to the air due to convection
end

% Making the thermal coefficient matrix A

% For m=1 Inlet side nodes
for i=1
   for j=1 %inner pipe flow
       Q(j,j)=-QL(Vflow,T0(j,1))-QR1(r1,r2,T0(j,1),T0(j+1,1),dL,Vflow);
       Q(j+1,j)=QR1(r1,r2,T0(j,1),T0(j+1,1),dL,Vflow);
   end
   for j=2 %outer pipe flow
       Q(j-1,j)=QR1(r1,r2,T0(j-1,1),T0(j,1),dL,Vflow);
       Q(j,j)=-QL(Vflow,T0(j,1))-QR1(r1,r2,T0(j-1,1),T0(j,1),dL,Vflow)...
           -QR22(r1,r2,T0(j,1),Tinf,dL,U,Vflow);
       Q(j,j+n)=QL(Vflow,T0(j+n,1));
   end
end

% For m= 2:m-1 Mid nodes
for i=2:1:m-1
   for j=1+(n*(i-1)) %inner pipe flow
       Q(j,j-n)=QL(Vflow,T0(j-n,1));
       Q(j,j)=-QL(Vflow,T0(j,1))-QR1(r1,r2,T0(j,1),T0(j+1,1),dL,Vflow);
       Q(j+1,j)=QR1(r1,r2,T0(j,1),T0(j+1,1),dL,Vflow);
   end
   for j=2+(n*(i-1)) %outer pipe flow
       Q(j-1,j)=QR1(r1,r2,T0(j-1,1),T0(j,1),dL,Vflow);
       Q(j,j)=-QL(Vflow,T0(j,1))-QR22(r1,r2,T0(j,1),Tinf,dL,U,Vflow)...
           -QR1(r1,r2,T0(j-1,1),T0(j,1),dL,Vflow);
       Q(j,j+n)=QL(Vflow,T0(j+n,1));
   end  
end

% For m=m The nodes where the water turns
for i=m
   for j=1+(n*(i-1)) %inner pipe flow
       Q(j,j-n)=QL(Vflow,T0(j-n,1)) ;
       Q(j,j)=-QL(Vflow,T0(j,1))-QR1(r1,r2,T0(j,1),T0(j+1,1),dL,Vflow) ;
       Q(j+1,j)=QL(Vflow,T0(j,1))+QR1(r1,r2,T0(j,1),T0(j+1,1),dL,Vflow) ;
   end
   for j=2+(n*(i-1)) %outer pipe flow
       Q(j-1,j)=QR1(r1,r2,T0(j-1,1),T0(j,1),dL,Vflow) ;
       Q(j,j)=-QL(Vflow,T0(j,1))-QR1(r1,r2,T0(j-1,1),T0(j,1),dL,Vflow)...
           -QR22(r1,r2,T0(j,1),Tinf,dL,U,Vflow) ;       
   end
end

T = Q\Qtot;         % calculating the resulting temperature vector T
T0=T;               %updating the new T0 vector for the next iteration

end


%% Prepearing results for plotting========================================

% Separating the result vector T into Ti(inner pipe) and To(outer pipe)
Ti(1)=Tin;     
s=1;
for p=1:n:m*n
  Ti(s+1)=T(p,1);
  To(s+1)=T(p+1,1);
  s=s+1;
end

%Estimating the outlet temperature from the last two temperature points
To(1)=To(2)+(To(2)-To(3)); 

% Calculaing Ts(surface temperature) from To(temperature outer pipe)
for i=1:m+1
    Ts(i)=((hw2(To(i),r1,r2,Vflow)*To(i))+(hair(Tinf,To(i),U,r2)*Tinf))...
        /((hair(Tinf,To(i),U,r2)+hw2(To(i),r1,r2,Vflow)));
end

% Making L vector for plotting
for l=1:m
    L(1)=0;
    L(l+1)=dL*l;
end

%change the unit from Kelvin to degree Celsius for plotting
Ts=Ts-273.15;
To=To-273.15; 
Ti=Ti-273.15; 
Tlim=Tlim-273.15; 

% DNV GL Rules for minimum surface temperature
Limmit=ones(1,length(L))*Tlim; 

%% Plotting===========================================================

plot1=plot(L,Ts,'r',L,To,'g',L,Ti,'b')%,L,Limmit,'c')
set(plot1,'LineWidth',2.5)
set(gca,'Fontsize',16)
leg=legend({'Temperature at the surface $$T_s$$'...
    ,'Temperature outer pipe flow $$T_o$$'...
    ,'Temperature inner pipe flow $$T_i$$'}...
    ,'box','off');%,'Surface temperature limmit by DNV GL')
set(leg,'Interpreter','latex','Fontsize',16);
xlabel('Length of pipe L [m]','Interpreter','latex','Fontsize',24)
ylabel('Temperature $$[^\circ C]$$','Interpreter','latex','Fontsize',24)
xlswrite('TiToTsC2.xlsx',[L' Ti' To' Ts']) %Temperature into Excel format